//
// This is the entry script for the CLI utility.
//

//region Imports
import {logHeading, padlog, log} from './util/logging'
import prompt from './util/prompt'
const c = require('chalk')
//endregion

process.on('unhandledRejection', e => {throw e})

function cli() {
  main().then(async () => {})
  return true
}

module.exports = cli

////////////////////////////////////////////////////////////////////////////////

const main = async function() {

  welcome()
  await confirmLocation(1)
  initPackageJson(2)
  await addGulpScriptToPackageJson(3)
  await createGulpfile(4)
  await installModules(5)
  gulpBootstrap(6)
  printHelp(7)

}

////////////////////////////////////////////////////////////////////////////////
// Steps
////////////////////////////////////////////////////////////////////////////////

// TODO
// - Use init-package-json.

function welcome(order) {

  log(c.bold('Welcome to the Babelator CLI'))
  log(c.grey(`- Babelator will first create a minimal npm module with gulp installed, and Babelator's gulp tasks available.`))
  log(c.grey(`- The rest of Babelator's bootstrapping occurs through gulp tasks by running \`gulp bootstrap\`.`))
  log(c.grey('- What about existing files? Babelator will confirm all changes to existing files making it super safe and easy to babelate existing projects.'))

}

async function confirmLocation(order) {

  logHeading(`${order}. Confirm module location`)

  const inquirer = require('inquirer')
  const cwd = require('cwd')

  const ans = await prompt([{type: 'confirm', name: 'confirm', message: `Confirm your module's location: '${cwd()}/package.json'`}])
  if (!ans.confirm) {
    log('Exiting')
    process.exit(1)
  }

}

function initPackageJson(order) {

  logHeading(`${order}. Create package.json if it does not exist`)

  const fs = require('fs')
  const cwd = require('cwd')
  const {spawnSync} = require('child_process')

  const packageJsonExists = fs.existsSync(cwd('package.json'))
  if (!packageJsonExists) {
    console.log('File `package.json` does not exist. A basic file will be created now. Don\'t worry, the contents will be iteratively modified further on.')
    log('=> npm init -y')
    // NOTE: -y - Use defaults and do not prompt user.
    spawnSync('npm', ['init', '-y'], {stdio: 'inherit'})
  } else {
    log('File `package.json` already exists.')
  }

}

async function addGulpScriptToPackageJson(order) {

  logHeading(`${order}. Add gulp command to package.json scripts hash`)

  const {promisify} = require('bluebird')
  const modifyPackageJson = require('./util/modifyPackageJson')
  const _ = require('lodash')
  await promisify(modifyPackageJson)(json => {
    _.set(json, 'scripts.gulp', './node_modules/gulp/bin/gulp.js')
    return json
  })

}

let useWebpackerator

async function createGulpfile(order) {

  logHeading(`${order}. Add Babelator tasks to gulpfile.js`)

  const {promisify} = require('bluebird')
  const replaceFile = require('./util/replaceFile')
  const cwd = require('cwd')
  const {confirm} = require('inquirer-promise')

  useWebpackerator = await confirm('Do you want to use webpack via webpackerator?')

  let gulpfileContents
  gulpfileContents = "var gulp = require('gulp')\n"
  if (useWebpackerator) gulpfileContents += "require('webpackerator/defaults/gulp')(gulp)\n"
  // babelator comes last so it can use all gulp files for bootstrap.
  gulpfileContents += "require('babelator')(gulp)\n"


  await promisify(replaceFile)(cwd('gulpfile.js'), gulpfileContents)

}

async function installModules(order) {

  // TODO(vjpr): Set to current versions from npm.
  let devDependencies = [
    ['babelator', '0.2.8'],
    ['gulp', 'gulpjs/gulp.git#4.0'],
    ['babel-preset-live', '0.0.4'],
  ]

  let dependencies = [
    ['babel-register', '^6'], // For `mocha:src` tests and if we add a `modules` dir.
    ['babel-runtime', '^5'],
    ['bluebird', '^3'],
    ['babel-polyfill', '^6'],
  ]

  //////////////////////////////////////////////////////////////////////////////

  if (useWebpackerator) {
    devDependencies = devDependencies.concat([
      ['webpackerator', '^0.1.0'],
      ['babel-preset-react-hmre', '1.1.1'],
      ['webpack-dev-server', '1.14.1'],
    ])
    dependencies = dependencies.concat([
      ['babel-loader', '^6'],
    ])
  }

  //////////////////////////////////////////////////////////////////////////////

  logHeading(`${order}. Add npm dependencies to package.json`)

  await require('npm-install-prompt').addNpmDependenciesToPackageJson({
    devDependencies,
    dependencies,
    reject: {
      devDependencies: ['babel-preset-react', 'babel-preset-es2015', 'babel-preset-stage-0'],
      dependencies: ['babel-preset-react', 'babel-preset-es2015', 'babel-preset-stage-0'],
    },
  })

  //////////////////////////////////////////////////////////////////////////////

  logHeading(`${order}. Install npm dependencies`)

  log(c.grey('- `npm install` takes some time. To speed up the process you can use `npm link` instead.'))
  log(c.grey('- For this to work you need to checkout the repo and `npm link` it.'))
  log(c.grey('  - E.g. `git checkout https://github.com/vjpr/babelator && npm install && npm link`'))
  log(c.grey('- For monorepos like babel you need to `npm link` the individual packages.'))
  log(c.grey('  - E.g. `git checkout https://github.com/babel/babel && cd packages/babel-preset-es2015 && npm install && npm link`'))
  log(c.grey('Tip: Use `npm_lazy` module to locally cache npm modules to speed up installs.'))
  log()

  await require('npm-install-prompt').installNpmDependencies({devDependencies, dependencies})

}

function gulpBootstrap(order) {

  const {spawnSync} = require('child_process')

  const cmd = 'npm run gulp bootstrap'

  log()
  log('=>', cmd)

  spawnSync('npm', ['run', 'gulp', 'bootstrap'], {stdio: 'inherit'})

}

function printHelp(order) {

  // TODO
  padlog('Bootstrapping complete!')

  log(c.bold.yellow('Getting started'))
  log()
  log('  npm run gulp test-watch ' + c.grey('Compiles and run tests on file change. Useful for development!'))
  log('  npm run gulp nodemon ' + c.grey('Run package.json main script with nodemon'))
  log('  npm run gulp release ' + c.grey('Interactive CLI to release new version on npm'))
  log('  npm-check -us ' + c.grey('Interactively update all your dependencies to their latest version'))
  log()

}
